@@ -34,6 +34,7 @@ typedef struct _LogProtoClient LogProtoClient;
 
 typedef struct _LogProtoClientOptions
 {
+  gboolean drop_input;
 } LogProtoClientOptions;
 
 typedef union _LogProtoClientOptionsStorage
@@ -52,6 +53,8 @@ typedef struct
   gpointer user_data;
 } LogProtoClientFlowControlFuncs;
 
+void log_proto_client_options_set_drop_input(LogProtoClientOptions *options, gboolean drop_input);
+
 void log_proto_client_options_defaults(LogProtoClientOptions *options);
 void log_proto_client_options_init(LogProtoClientOptions *options, GlobalConfig *cfg);
 void log_proto_client_options_destroy(LogProtoClientOptions *options);
@@ -62,12 +65,14 @@ struct _LogProtoClient
   const LogProtoClientOptions *options;
   LogTransport *transport;
   /* FIXME: rename to something else */
-  gboolean (*prepare)(LogProtoClient *s, gint *fd, GIOCondition *cond);
+  gboolean (*prepare)(LogProtoClient *s, gint *fd, GIOCondition *cond, gint *timeout);
   LogProtoStatus (*post)(LogProtoClient *s, LogMessage *logmsg, guchar *msg, gsize msg_len, gboolean *consumed);
+  LogProtoStatus (*process_in)(LogProtoClient *s);
   LogProtoStatus (*flush)(LogProtoClient *s);
   gboolean (*validate_options)(LogProtoClient *s);
   gboolean (*handshake_in_progess)(LogProtoClient *s);
   LogProtoStatus (*handshake)(LogProtoClient *s);
+  gboolean (*restart_with_state)(LogProtoClient *s, PersistState *state, const gchar *persist_name);
   void (*free_fn)(LogProtoClient *s);
   LogProtoClientFlowControlFuncs flow_control_funcs;
 };
@@ -120,9 +125,9 @@ log_proto_client_handshake(LogProtoClient *s)
 }
 
 static inline gboolean
-log_proto_client_prepare(LogProtoClient *s, gint *fd, GIOCondition *cond)
+log_proto_client_prepare(LogProtoClient *s, gint *fd, GIOCondition *cond, gint *timeout)
 {
-  return s->prepare(s, fd, cond);
+  return s->prepare(s, fd, cond, timeout);
 }
 
 static inline LogProtoStatus
@@ -134,6 +139,21 @@ log_proto_client_flush(LogProtoClient *s)
     return LPS_SUCCESS;
 }
 
+static inline LogProtoStatus
+log_proto_client_process_in(LogProtoClient *s)
+{
+  if (s->process_in)
+    return s->process_in(s);
+  else if (s->flush)
+    /*
+     * In some clients, flush is used for input processing, but it should not be.
+     * Fix these clients, than remove the flush call here.
+     */
+    return s->flush(s);
+  else
+    return LPS_SUCCESS;
+}
+
 static inline LogProtoStatus
 log_proto_client_post(LogProtoClient *s, LogMessage *logmsg, guchar *msg, gsize msg_len, gboolean *consumed)
 {
@@ -153,6 +173,14 @@ log_proto_client_reset_error(LogProtoClient *s)
   s->status = LPS_SUCCESS;
 }
 
+static inline gboolean
+log_proto_client_restart_with_state(LogProtoClient *s, PersistState *state, const gchar *persist_name)
+{
+  if (s->restart_with_state)
+    return s->restart_with_state(s, state, persist_name);
+  return FALSE;
+}
+
 gboolean log_proto_client_validate_options(LogProtoClient *self);
 void log_proto_client_init(LogProtoClient *s, LogTransport *transport, const LogProtoClientOptions *options);
 void log_proto_client_free(LogProtoClient *s);
@@ -164,6 +192,7 @@ void log_proto_client_free_method(LogProtoClient *s);
   {                                                                     \
     static LogProtoClientFactory proto = {                              \
       .construct = prefix ## _client_new,                               \
+      .stateful  = FALSE,                                               \
     };                                                                  \
     return &proto;                                                      \
   }
@@ -187,6 +216,9 @@ typedef struct _LogProtoClientFactory LogProtoClientFactory;
 struct _LogProtoClientFactory
 {
   LogProtoClient *(*construct)(LogTransport *transport, const LogProtoClientOptions *options);
+  gint default_inet_port;
+  gboolean use_multitransport;
+  gboolean stateful;
 };
 
 static inline LogProtoClient *
@@ -196,6 +228,12 @@ log_proto_client_factory_construct(LogProtoClientFactory *self, LogTransport *tr
   return self->construct(transport, options);
 }
 
+static inline gboolean
+log_proto_client_factory_is_proto_stateful(LogProtoClientFactory *self)
+{
+  return self->stateful;
+}
+
 LogProtoClientFactory *log_proto_client_get_factory(PluginContext *context, const gchar *name);
 
 #endif
